package com.ahai.vgk_game_services

import android.app.Activity
import android.content.Intent
import android.util.Log
import android.util.Pair
import android.view.Gravity
import com.google.android.gms.auth.api.Auth
import com.google.android.gms.auth.api.signin.GoogleSignIn
import com.google.android.gms.auth.api.signin.GoogleSignInAccount
import com.google.android.gms.auth.api.signin.GoogleSignInClient
import com.google.android.gms.auth.api.signin.GoogleSignInOptions
import com.google.android.gms.games.Games
import com.google.android.gms.drive.Drive
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import io.flutter.plugin.common.PluginRegistry.ActivityResultListener
import java.io.IOException

private const val CHANNEL_NAME = "vgk_game_services"
private const val RC_SIGN_IN = 9000

class VgkGameServicesPlugin(private var activity: Activity? = null) : FlutterPlugin, MethodCallHandler, ActivityAware, ActivityResultListener {

  private var googleSignInClient: GoogleSignInClient? = null
  private var googleSignInAccount: GoogleSignInAccount? = null
  private var activityPluginBinding: ActivityPluginBinding? = null
  private var channel: MethodChannel? = null
  private var pendingOperation: PendingOperation? = null
  private lateinit var savedGamesController: SavedGamesController
  private lateinit var achievementsController: AchievementsController
  private lateinit var leaderboardsController: LeaderboardsController
  
  companion object {
    @JvmStatic
    fun registerWith(registrar: PluginRegistry.Registrar) {
      val channel = MethodChannel(registrar.messenger(), CHANNEL_NAME)
      val plugin = VgkGameServicesPlugin(registrar.activity())
      channel.setMethodCallHandler(plugin)
      registrar.addActivityResultListener(plugin)
    }
  }
  private fun signOut(result: Result) {
    val activity = this.activity!!
    googleSignInClient?.signOut()?.addOnCompleteListener(activity) { task ->
      if (task.isSuccessful) {
        googleSignInClient = null
        googleSignInAccount = null
        pendingOperation = null
        result.success("success")
      } else {
        result.success("Error")
      }
    }
  }
  private fun getSignIn(result: Result) {
    val activity = activity ?: return
    val builder = GoogleSignInOptions.Builder(
            GoogleSignInOptions.DEFAULT_GAMES_SIGN_IN)
    builder.requestScopes(Games.SCOPE_GAMES_SNAPSHOTS, Drive.SCOPE_APPFOLDER)
    googleSignInClient = GoogleSignIn.getClient(activity, builder.build())
    googleSignInClient?.silentSignIn()?.addOnCompleteListener { response ->
      pendingOperation = PendingOperation(Methods.getSignIn, result)
      if (response.isSuccessful) {
        googleSignInAccount = response.result
        handleSignInResult(googleSignInAccount!!)
      } else {
        Log.e("error", "signInError", response.exception)
        Log.i("ExplicitSignIn", "Trying explicit sign in")
        explicitSignIn()
      }
    }
  }

  private fun explicitSignIn() {
    val activity = activity ?: return
    val builder = GoogleSignInOptions.Builder(
            GoogleSignInOptions.DEFAULT_GAMES_SIGN_IN)
            .requestEmail()
    builder.requestScopes(Games.SCOPE_GAMES_SNAPSHOTS, Drive.SCOPE_APPFOLDER)
    googleSignInClient = GoogleSignIn.getClient(activity, builder.build())
    activity.startActivityForResult(googleSignInClient?.signInIntent, RC_SIGN_IN)
  }

  private fun handleSignInResult(googleSignInAccount: GoogleSignInAccount) {
    val activity = this.activity!!
    savedGamesController = SavedGamesController(Games.getSnapshotsClient(activity, googleSignInAccount))
    achievementsController = AchievementsController(Games.getAchievementsClient(activity, googleSignInAccount), activity)
    leaderboardsController = LeaderboardsController(Games.getLeaderboardsClient(activity, googleSignInAccount), activity)

    val gamesClient = Games.getGamesClient(activity, GoogleSignIn.getLastSignedInAccount(activity)!!)
    gamesClient.setViewForPopups(activity.findViewById(android.R.id.content))
    gamesClient.setGravityForPopups(Gravity.TOP or Gravity.CENTER_HORIZONTAL)

    val playersClient = Games.getPlayersClient(activity!!, googleSignInAccount)
    playersClient.currentPlayer?.addOnSuccessListener { currentPlayer ->
      val successMap = mapOf(
              "response" to "success",
              "message" to "player connect to game center",
              "id" to currentPlayer.playerId,
              "displayName" to currentPlayer.displayName)

      pendingOperation!!.result.success(successMap)
      pendingOperation = null

    }?.addOnFailureListener {
      pendingOperation!!.result.success( "Error")
      pendingOperation = null
    }
  }

  private fun showLoginErrorIfNotLoggedIn(result: Result) {
    result.success("Error")
  }

  override fun onAttachedToEngine(binding: FlutterPlugin.FlutterPluginBinding) {
    setupChannel(binding.binaryMessenger)
  }

  override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
    teardownChannel()
  }

  private fun setupChannel(messenger: BinaryMessenger) {
    channel = MethodChannel(messenger, CHANNEL_NAME)
    channel?.setMethodCallHandler(this)
  }

  private fun teardownChannel() {
    channel?.setMethodCallHandler(null)
    channel = null
  }

  private fun disposeActivity() {
    activityPluginBinding?.removeActivityResultListener(this)
    activityPluginBinding = null
  }

  override fun onDetachedFromActivity() {
    disposeActivity()
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    onAttachedToActivity(binding)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activityPluginBinding = binding
    activity = binding.activity
    binding.addActivityResultListener(this)
  }

  override fun onDetachedFromActivityForConfigChanges() {
    onDetachedFromActivity()
  }

  private class PendingOperation constructor(val method: String, val result: Result)

  private fun finishPendingOperationWithError(errorMessage: String) {
    Log.i(pendingOperation!!.method, "error")
    pendingOperation!!.result.error("error", errorMessage, null)
    pendingOperation = null
  }

  override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent): Boolean {
    if (requestCode == RC_SIGN_IN) {
      val result = Auth.GoogleSignInApi.getSignInResultFromIntent(data)
      val signInAccount = result?.signInAccount
      if (result?.isSuccess == true && signInAccount != null) {
        handleSignInResult(signInAccount)
      } else {
        var message = result?.status?.statusMessage ?: ""
        if (message.isEmpty()) {
          message = "Something went wrong " + result?.status
        }
        finishPendingOperationWithError(message)
      }
      return true
    }
    return false
  }

  override fun onMethodCall(call: MethodCall, result: Result) {
    when (call.method) {

      Methods.getSignIn -> {
        getSignIn(result)
      }

      Methods.signOut -> {
        signOut(result)
      }

      Methods.saveSnapShot -> {
        val name = call.argument<String>("name") ?: ""
        val description = call.argument<String>("description") ?: ""
        val snapshotData = call.argument<String>("snapshotData") ?: ""
        savedGamesController.saveSnapShot(name, description, snapshotData, result)
      }

      Methods.loadSnapShot -> {
        val name = call.argument<String>("name") ?: ""
        savedGamesController.loadSnapShot(name, result)
      }

      Methods.unlockAchievement -> {
        val achievementName = call.argument<String>("name") ?: ""
        achievementsController.unlockAchievement(achievementName, result)
      }

      Methods.revealAchievement -> {
        val achievementName = call.argument<String>("name") ?: ""
        achievementsController.revealAchievement(achievementName, result)
      }

      Methods.setAchievementSteps -> {
        val achievementName = call.argument<String>("name") ?: ""
        val steps = call.argument<Int>("steps") ?: 0
        achievementsController.setAchievementSteps(achievementName, steps, result)
      }

      Methods.showAchievements -> {
        achievementsController.showAchievements(result)
      }

      Methods.loadAchievementInfo -> {
        achievementsController.loadAchievementInfo(true, result)
      }

      Methods.setPercentAchievement -> {
        val achievementName = call.argument<String>("id") ?: ""
        val percent = call.argument<Int>("percent") ?: 0
        achievementsController.setPercentAchievement(achievementName, percent, result)
      }

      Methods.submitScore -> {
        val id = call.argument<String>("id") ?: ""
        val score = call.argument<Int>("score") ?: 0
        leaderboardsController.submitScore(id, score, result)
      }

      Methods.showLeaderboard -> {
        val id = call.argument<String>("id") ?: ""
        leaderboardsController.showLeaderboard(id, result)
      }

      Methods.showAllLeaderboards -> {
        leaderboardsController.showAllLeaderboards(result)
      }

      else -> result.notImplemented()
    }
  }
}

private object Methods {
  const val getSignIn = "getSignIn"
  const val signOut = "signOut"
  const val saveSnapShot = "saveSnapShot"
  const val loadSnapShot = "loadSnapShot"
  const val unlockAchievement = "unlockAchievement"
  const val revealAchievement =  "revealAchievement"
  const val setAchievementSteps = "setAchievementSteps"
  const val showAchievements = "showAchievements"
  const val loadAchievementInfo = "loadSnapShot"
  const val setPercentAchievement = "setPercentAchievement"
  const val submitScore = "submitScore"
  const val showLeaderboard = "showLeaderboard"
  const val showAllLeaderboards = "showAllLeaderboards"
}